"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GithubApi = void 0;
const rest_1 = require("@octokit/rest");
const lodash_1 = __importDefault(require("lodash"));
class GithubApi {
    constructor(inputs) {
        this.inputs = inputs;
    }
    async getBranch(repo, branch) {
        const apiClient = await this.prepareApiClient();
        const [repoOwner, repoName] = this.getRepoOwnerAndName(repo);
        const response = await apiClient.repos.getBranch({
            owner: repoOwner,
            repo: repoName,
            branch,
        });
        return response.data;
    }
    async getCommit(repo, branch, revision) {
        const apiClient = await this.prepareApiClient();
        const [repoOwner, repoName] = this.getRepoOwnerAndName(repo);
        const response = await apiClient.repos.getCommit({
            owner: repoOwner,
            repo: repoName,
            ref: revision,
        });
        return response.data;
    }
    async searchForPullRequests(repo, branch) {
        const apiClient = await this.prepareApiClient();
        console.log(`Looking for PRs from "${repo}" repo and "${branch}" branch`);
        const response = await apiClient.search.issuesAndPullRequests({ q: `head:${branch}+type:pr+repo:${repo}` });
        return response.data;
    }
    async getCommitsInfo(repo, pullRequestId) {
        const apiClient = await this.prepareApiClient();
        const [repoOwner, repoName] = this.getRepoOwnerAndName(repo);
        const commitsByUserLimit = this.inputs.commitsByUserLimit;
        const committersMap = {};
        const commitsByUser = {};
        let firstCommitDate;
        let lastCommitDate;
        let page = 1;
        while (true) {
            const commits = await apiClient.pulls.listCommits({
                owner: repoOwner,
                repo: repoName,
                pull_number: pullRequestId,
                page,
            });
            if (commits.data.length === 0) {
                break;
            }
            if (page === 1) {
                firstCommitDate = commits.data[0].commit.author?.date;
            }
            lastCommitDate = lodash_1.default.last(commits.data)?.commit.author?.date;
            for (const commit of commits.data) {
                const userName = commit.author?.login || lodash_1.default.get(commit, 'commit.committer.name');
                if (!userName) {
                    continue;
                }
                committersMap[userName] = {
                    userName,
                    avatar: commit.author?.avatar_url,
                };
                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = [];
                }
                commitsByUser[userName].push({
                    url: commit.html_url,
                    userName,
                    sha: commit.sha,
                    message: commit.commit.message,
                    commitDate: commit.commit.author?.date
                });
            }
            page++;
        }
        for (const userName of Object.keys(commitsByUser)) {
            commitsByUser[userName] = lodash_1.default.takeRight(commitsByUser[userName], commitsByUserLimit);
        }
        return {
            committers: lodash_1.default.values(committersMap),
            commits: lodash_1.default.flatten(lodash_1.default.values(commitsByUser)),
            firstCommitDate,
            lastCommitDate
        };
    }
    async prepareApiClient() {
        const githubToken = this.inputs.githubToken;
        const githubApiHost = this.inputs.githubApiHost || 'api.github.com';
        const githubApiPathPrefix = this.inputs.githubApiPathPrefix || '/';
        const octokit = new rest_1.Octokit({
            auth: githubToken,
            baseUrl: this.buildRequestUrl(githubApiHost, githubApiPathPrefix)
        });
        return octokit;
    }
    buildRequestUrl(apiHost, apiPathPrefix) {
        try {
            let pathPrefix = apiPathPrefix.replace(/\/$/, '');
            if (pathPrefix && pathPrefix[0] !== '/') {
                pathPrefix = `/${pathPrefix}`;
            }
            let host = apiHost.replace(/\/$/, '');
            if (!/^http(s?)/.test(host)) {
                host = `https://${host}`;
            }
            return `${host}${pathPrefix}`;
        }
        catch (error) {
            console.error(error);
            throw new Error(`github api host is not valid`);
        }
    }
    getRepoOwnerAndName(repo) {
        const [owner, ...name] = repo.split('/');
        return [owner, name.join('/')];
    }
}
exports.GithubApi = GithubApi;
//# sourceMappingURL=github.api.js.map