"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitlabApi = void 0;
const node_1 = require("@gitbeaker/node");
const lodash_1 = __importDefault(require("lodash"));
const request_promise_1 = __importDefault(require("request-promise"));
class GitlabApi {
    constructor(inputs) {
        this.inputs = inputs;
        this.api = new node_1.Gitlab({
            host: inputs.gitlabHost,
            token: inputs.gitlabToken,
        });
    }
    async getCommitsInfo(repo, pr) {
        const commits = await this.api.MergeRequests.commits(repo, pr.iid);
        const avatarsByName = await this._getParticipantsAvatars(repo, pr.iid);
        if (commits && commits.length > 0) {
            const firstCommitDate = commits[0].created_at;
            const lastCommitDate = lodash_1.default.last(commits).created_at;
            const commitsByUser = {};
            const committersMap = {};
            for (const commit of commits) {
                const avatar = await this._getAvatarFromCommit(avatarsByName, commit);
                const userName = commit.committer_email;
                committersMap[userName] = {
                    userName,
                    avatar,
                };
                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = [];
                }
                commitsByUser[userName].push({
                    url: commit.web_url,
                    userName,
                    sha: commit.id,
                    message: commit.message,
                    commitDate: commit.created_at
                });
            }
            return {
                committers: lodash_1.default.values(committersMap),
                commits: lodash_1.default.flatten(lodash_1.default.values(commitsByUser)),
                firstCommitDate,
                lastCommitDate
            };
        }
        return null;
    }
    async getPullRequests(repo, branch) {
        return await this.api.MergeRequests.all({ projectId: repo, state: 'all', scope: 'all', sourceBranch: branch });
    }
    async getBranch(repo, branch) {
        return await this.api.Branches.show(repo, branch);
    }
    async _getAvatarForEmail(email) {
        try {
            return await (0, request_promise_1.default)({
                method: 'GET',
                uri: `${this.inputs.gitlabHost}/api/v4/avatar?email=${email}&size=32`,
                json: true
            });
        }
        catch (error) {
            console.log(error);
            return null;
        }
    }
    async _getParticipantsAvatars(repo, prIid) {
        const participants = await this.api.MergeRequests.participants(repo, prIid);
        const avatarsByUser = {};
        lodash_1.default.forEach(participants, participant => {
            avatarsByUser[participant.name] = participant.avatar_url;
        });
        return avatarsByUser;
    }
    async _getAvatarFromCommit(avatarsByName, commit) {
        const avatar = avatarsByName[commit.committer_name];
        if (!avatar) {
            const result = await this._getAvatarForEmail(commit.committer_email);
            return lodash_1.default.get(result, 'avatar_url');
        }
        return avatar;
    }
}
exports.GitlabApi = GitlabApi;
//# sourceMappingURL=gitlab.api.js.map