"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reportImageInfo = void 0;
const lodash_1 = __importDefault(require("lodash"));
const registries_1 = require("../api/registries");
const codefresh_1 = require("../api/codefresh");
const types_1 = require("./types");
async function reportImageInfo(inputs, notify) {
    console.log('starting image reporter library');
    const image = inputs.imageName;
    const registryClient = new registries_1.RegistryClient(inputs);
    const client = await registryClient.getRegistryClient(image);
    console.log(`using ${client.constructor.name} client`);
    const workflowName = inputs.workflow?.name;
    const workflowUrl = inputs.workflow?.workflowUrl;
    const logsUrl = inputs.workflow?.logsUrl;
    const registry = client.repoTag(image);
    const manifest = await registry.getManifest();
    console.log(`image manifest has been fetched:`, '\n', manifest);
    const config = await registry.getConfig(manifest);
    console.log(`image config has been fetched:`, '\n', config);
    const repositoryName = lodash_1.default.get(registryClient.parseImageName(image), 'repository');
    const imageLink = `${inputs.codefresh.host}/2.0/images/${encodeURIComponent(repositoryName)}/${manifest.config.digest}/${encodeURIComponent(image)}`;
    const size = registryClient.getRegistryImageSize(manifest);
    try {
        await notify(types_1.NOTIFICATIONS.IMAGE_NAME, image);
    }
    catch (error) {
        console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_NAME}". ${error.message}`);
    }
    try {
        await notify(types_1.NOTIFICATIONS.IMAGE_SHA, lodash_1.default.get(manifest, 'config.digest'));
    }
    catch (error) {
        console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_SHA}". ${error.message}`);
    }
    try {
        await notify(types_1.NOTIFICATIONS.IMAGE_LINK, imageLink);
    }
    catch (error) {
        console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_LINK}". ${error.message}`);
    }
    const { host, apiKey } = inputs.codefresh;
    const codefreshAPI = new codefresh_1.CodefreshAPI(host, apiKey);
    const configCreated = config.created || new Date().toISOString();
    const imageBinaryVars = {
        imageBinary: {
            id: manifest.config.digest,
            created: configCreated,
            imageName: image,
            dockerFile: await registryClient.buildDockerfile(),
            size: size,
            os: config.os,
            architecture: config.architecture,
            workflowName: workflowName,
            workflowUrl,
            logsUrl,
        }
    };
    console.log('binaryMutation payload:', imageBinaryVars.imageBinary);
    const binaryResult = await codefreshAPI.createImageBinary(imageBinaryVars);
    console.log(JSON.stringify(binaryResult, null, 2));
    const imageLayersVars = {
        imageLayers: {
            image: manifest.config.digest,
            layers: registryClient.getLayersFromHistory(config.history, manifest.layers),
        }
    };
    console.log('ImageLayers Mutation payload:', JSON.stringify(imageLayersVars.imageLayers, null, 2));
    const imageLayersResult = await codefreshAPI.createImageLayers(imageLayersVars)
        .catch((layersError) => {
        console.error(layersError);
    });
    console.log(JSON.stringify(imageLayersResult, null, 2));
    const imageRegistryVars = {
        imageRegistry: {
            binaryId: binaryResult.createImageBinary.id,
            imageName: image,
            repoDigest: manifest.config.repoDigest,
            created: configCreated
        }
    };
    console.log('registryMutation payload:', imageRegistryVars.imageRegistry);
    const registryResult = await codefreshAPI.createImageRegistry(imageRegistryVars);
    console.log(JSON.stringify(registryResult, null, 2));
    console.log('image reported successfully');
}
exports.reportImageInfo = reportImageInfo;
//# sourceMappingURL=index.js.map