import _ from 'lodash'
import rp from 'request-promise'

import { CommitInfo, GitEnricherInputs } from '../../../image-enricher-git-info/types'
import { GerritChange, GerritCommit } from './types'

/**
 * To prevent against Cross Site Script Inclusion (XSSI) attacks, the JSON response body starts with a magic
 * prefix line that must be stripped before feeding the rest of the response body to a JSON parser:
 */
const MAGIC_PREFIX = ')]}\''

export class GerritApi {

    private inputs: GitEnricherInputs

    constructor(inputs: GitEnricherInputs) {
        this.inputs = inputs
    }

    async getChange(changeId, repo: string): Promise<GerritChange | null>  {
        console.log(`Search changes by changeId: ${changeId}`)
        const changes = await this.sendRequest(
            `changes/?q=change:${changeId}+repo:${repo}&o=ALL_REVISIONS&o=ALL_COMMITS&o=WEB_LINKS&o=MESSAGES&o=CURRENT_COMMIT`)
        if (changes.length > 0) {
            return changes[0]
        }
        return null
    }

    async getCommitsInfo(pr: GerritChange): Promise<CommitInfo | null> {
        const revisions = pr.revisions ? Object.keys(pr.revisions): []
        if (revisions.length > 0) {
            const firstCommit = revisions[0]
            const lastCommit = _.last<any>(revisions)
            const firstCommitDate = _.get(pr, `revisions[${firstCommit}].created`)
            const lastCommitDate =  _.get(pr, `revisions[${lastCommit}].created`)
            const commitsByUser = {}
            const committersMap = {}
            for (const revision of revisions) {
                const commit: GerritCommit = _.get(pr, `revisions[${revision}].commit`)
                const userName = commit.committer.name
                const avatar = await this.getAvatar(userName)
                committersMap[userName] = {
                    userName,
                    avatar,
                }
                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = []
                }
                commitsByUser[userName].push({
                    url: `${this.inputs.gerritHost}${commit.web_links[0].url}`,
                    userName,
                    sha: revision,
                    message: commit.message,
                    commitDate: commit.committer.date
                })
            }
            return {
                committers: _.values(committersMap),
                commits: _.flatten(_.values(commitsByUser)),
                firstCommitDate,
                lastCommitDate
            } as CommitInfo
        }
        return null
    }

    private async getAvatar(username: string): Promise<string|null> {
        try {
            const result = await this.sendRequest(`/accounts/${username}/avatar?s=20`)
            return result
        } catch {
            return null
        }
    }

    private async sendRequest(query: string): Promise<any> {
        const uri = this.buildRequestUrl(this.inputs.gerritHost)
        const result = await rp({
            method: 'GET',
            uri: `${uri}/${query}`, //&q=is:private
            auth: {
                user: this.inputs.gerritUsername,
                pass: this.inputs.gerritPassword
            },
            json: true
        })
        if (!result.startsWith(MAGIC_PREFIX)) {
            throw new Error(`magic prefix not found: ${result.slice(0, 100)}`)
        }
        try {
            return JSON.parse(result.slice(MAGIC_PREFIX.length))
        } catch (error) {
            throw new Error(`invalid response: ${error.message}`)
        }
    }

    private buildRequestUrl(apiHost) {
        try {
            // Sanitizing URL for supporting all formats existing in DB
            // Normalize parts of url in format: (host)(/pathPrefix)
            let host = apiHost.replace(/\/$/, '') // remove the last slash
            if (!/^http(s?)/.test(host)) {
                host = `https://${host}`
            }

            return `${host}/a`
        } catch (error) {
            console.error(error)
            throw new Error(`gerrit host is not valid`)
        }
    }
}