import _ from 'lodash'

import { GerritApi } from './gerrit.api'
import { BranchInfo, GitEnricherInputs, PRInfo } from '../../../image-enricher-git-info/types'

const CHANGE_ID_KEYWORD = 'change-id:'

export class GerritService {
    private gerritApi
    private inputs

    constructor(inputs: GitEnricherInputs) {
        this.gerritApi = new GerritApi(inputs)
        this.inputs = inputs
    }

    static parseChangeId(gerritChangeId?: string): string {
        if (!gerritChangeId) {
            throw new Error('Gerrit changeId is not set')
        }
        // parse changeId from commit message using regex
        // support any commit message that contain Change-Id:
        if (gerritChangeId.toLowerCase().includes(CHANGE_ID_KEYWORD)) {
            const regexpResult = new RegExp('(?<=Change-Id:).*', 'i').exec(gerritChangeId)
            if (!_.isArray(regexpResult)||!_.isEmpty(regexpResult)) {
                throw new Error('Not found changeId')
            }
            return _.get(regexpResult, '[0]', '').replace( /\s/g, '')
        }
        return gerritChangeId
    }

    async getPullRequestsWithCommits(repo: string, changeId: string): Promise<PRInfo[]> {
        try {
            const change = await this.gerritApi.getChange(changeId, repo)
            const info = await this.gerritApi.getCommitsInfo(change)
            const result: PRInfo = {
                    ...info,
                    prDate: change.updated || change.created,
                    number: change._number,
                    url: `${this.inputs.gerritHost}/c/${repo}/+/${change._number}`,
                    title: change.subject,
                    targetBranch: change.branch
                } as PRInfo
            return [ result ]
        } catch (error) {
            throw new Error(`failed to get pull requests: ${error.message}`)
        }
    }

    async getBranch(repo: string, changeId: string): Promise<BranchInfo| null> {
        const gerritChangeInfo = await this.gerritApi.getChange(changeId, repo)
        if (!gerritChangeInfo) {
            return null
        }
        const currentRevision = gerritChangeInfo?.revisions[gerritChangeInfo?.current_revision]
        return {
            name: gerritChangeInfo.branch,
            commit: currentRevision.commit.message,
            commitMsg: currentRevision.commit.message,
            commitURL: `${this.inputs.gerritHost}${currentRevision.commit.web_links[0].url}`,
            committerUsername: currentRevision.commit.committer.name || currentRevision.commit.committer.email
        } as BranchInfo
    }
}
